const getElementName = path =>
  path.parent &&
  path.parent.openingElement &&
  path.parent.openingElement.name &&
  path.parent.openingElement.name.name;

function isI18n(path) {
  const callee = path.parentPath.node.callee;
  if (callee) {
    if (callee.type === "Import") return false;

    if (callee.type === "MemberExpression") {
      const complexCalleeName = `${callee.object.name}.${callee.property.name}`;
      return complexCalleeName === "i18n._";
    }
  }
  return false;
}

const StringMod = t =>
  function(path) {
    const value = path.node.value && path.node.value.trim();
    if (isI18n(path)) {
      path.parentPath.replaceWith(t.stringLiteral(value));
    }
  };

const TemplateMod = t =>
  function(path) {
    const parent = path.parent;
    const tagName = parent && parent.tag && parent.tag.name;
    if (t.isTaggedTemplateExpression(parent) && tagName === "t") {
      path.parentPath.parentPath.replaceWith(path);
    }
  };

const JSXTextMod = t =>
  function(path) {
    if (getElementName(path) === "Trans") {
      path.parentPath.replaceWith(
        t.jsxElement(
          t.jsxOpeningElement(t.jsxIdentifier(""), []),
          t.jsxClosingElement(t.jsxIdentifier("")),
          path.parent.children
        )
      );
    }
  };

function stringLiteralReplace(_ref) {
  const t = _ref.types;

  return {
    manipulateOptions(opts, parserOpts) {
      parserOpts.plugins.push("classProperties");
      parserOpts.plugins.push("optionalChaining");
      parserOpts.plugins.push("objectRestSpread");
      parserOpts.plugins.push("nullishCoalescingOperator");
      parserOpts.plugins.push("numericSeparator");
    },
    visitor: {
      Program(path) {
        path.traverse({
          TemplateLiteral: TemplateMod(t),
          StringLiteral: StringMod(t),
          JSXText: JSXTextMod(t)
        });
      }
    }
  };
}

module.exports = stringLiteralReplace;
